from django.db import models
from reqman.apps.account.models import User

from pygments.formatters.html import HtmlFormatter
from pygments import highlight

from reqman.apps.reqtool.rest_api.services.parse_ontologies import *


MAIN_CHOICES = ( ("",""),   ("M1", "M1 : system/function shall [not] set [<quantifier>] item [to stateValue]"), 
                            ("M2", "M2 : system/function shall [not] set state to stateValue"), 
                            ("M3", "M3 : system/function shall [not] send [<quantifier>] item"),
                            ("M4", "M4 : system/function shall [not] receive [<quantifier>] item"),
                            ("M5", "M5 : system/function shall [not] ingest flow"),
                            ("M6", "M6 : system/function shall [not] emit flow"),
                            ("M7", "M7 : system shall [not] perform function"),
                            ("M8", "M8 : function shall [not] invoke function"),
                            ("M9", "M9 : system shall [not] present interface [to system]"),
                            ("M10", "M10 : system/interface shall [not] transfer flow/item"),
                            ("M11", "M11 : system shall [not] interact with system [using <connection>] [as dened in <text>]"),
                            ("M12", "M12 : system shall [not] have state state [with values stateSet]"),
                            ("M13", "M13 : system state stateValue shall [not] have substate state [with values stateSet]"),
                            ("M14", "M14 : system item shall [not] take values from stateSet"),
                            ("M15", "M15 : system item shall [not] be composed from item"),
                            ("M16", "M16 : system shall [not] contain system")
                        )

#to be updated with domains from DSO
#SYSTEM_CHOICES = (("SYSTEM", "SYSTEM"), ("SYSTEM2", "SYSTEM2"))
SYSTEM_CHOICES = find_extra_keywords("SAO:System")
FUNCTION_CHOICES = (("FUNCTION", "FUNCTION"), ("FUNCTION2", "FUNCTION2"))
SHALL_CHOICES = ( ("shall", "SHALL"), ("shall not", "SHALL NOT"))
QUANTIFIER_CHOICES = ( ("none","NONE"), ("all", "ALL"), ("only", "ONLY"),
                                ("more than", "MORE THAN"), ("less than", "LESS THAN"), ("exactly","EXACTLY"), 
                                ("at least", "AT LEAST"), ("at most", "AT MOST")
                     )
NUMBER_UNITS_CHOICES = (("",""), ("meters","METERS"), ("kilometers","KILOMETERS"), ("volt","VOLT"))
ITEM_CHOICES = (("ITEM", "ITEM"), ("ITEM2", "ITEM2") )
STATE_CHOICES = (("state", "STATE"), ("state2", "STATE2"))
STATE_VALUE_CHOICES = (("enabled", "ENABLED"), ("disabled", "DISABLED"))
FLOW_CHOICES = (("flow","FLOW"), ("flow2","FLOW2"))
INTERFACE_CHOICES = (("interface","INTERFACE"), ("interface2","INTERFACE2"))
CONNECTION_CHOICES = (("connection","CONNECTION"), ("connection2","CONNECTION2"))
SYSTEM_INTERFACE_CHOICES = SYSTEM_CHOICES + INTERFACE_CHOICES
FLOW_ITEM_CHOICES = FLOW_CHOICES + ITEM_CHOICES
SYSTEM_OR_FUNCTION_CHOICES = SYSTEM_CHOICES + FUNCTION_CHOICES



def create_main(instance):
    '''Create the Main Model for the Boilerplate'''
    try :
        main = Main.objects.get(boilerplate_of_main = instance, main_owner = instance.owner)
        if instance.has_main:
            return
        main = Main.objects.get(boilerplate_of_main = instance, main_owner = instance.owner)
        main.delete()
    except Main.DoesNotExist:
        if instance.has_main:
            main = Main(boilerplate_of_main = instance, main_owner = instance.owner)
            main.save()


def create_m1(instance, now_main, prev_main):
    try :
        main = Main_M1.objects.get(main_owner_of_m1 = instance, boilerplate_of_m1 = instance.boilerplate_of_main)
        if now_main == prev_main:
            return
        main = Main_M1.objects.get(main_owner_of_m1 = instance, boilerplate_of_m1 = instance.boilerplate_of_main)
        main.delete()
        if now_main == MAIN_CHOICES[1][0]:
            main = Main_M1(main_owner_of_m1 = instance, boilerplate_of_m1 = instance.boilerplate_of_main)
            main.save()
    except Main_M1.DoesNotExist:
        if now_main == MAIN_CHOICES[1][0]:
            main = Main_M1(main_owner_of_m1 = instance, boilerplate_of_m1 = instance.boilerplate_of_main)
            main.save()


def create_m2(instance, now_main, prev_main):
    try :
        main = Main_M2.objects.get(main_owner_of_m2 = instance, boilerplate_of_m2 = instance.boilerplate_of_main)
        if now_main == prev_main:
            return
        main = Main_M2.objects.get(main_owner_of_m2 = instance, boilerplate_of_m2 = instance.boilerplate_of_main)
        main.delete()
        if now_main == MAIN_CHOICES[2][0]:
            main = Main_M2(main_owner_of_m2 = instance, boilerplate_of_m2 = instance.boilerplate_of_main)
            main.save()
    except Main_M2.DoesNotExist:
        if now_main == MAIN_CHOICES[2][0]:
            main = Main_M2(main_owner_of_m2 = instance, boilerplate_of_m2 = instance.boilerplate_of_main)
            main.save()


def create_m3(instance, now_main, prev_main):
    try :
        main = Main_M3.objects.get(main_owner_of_m3 = instance, boilerplate_of_m3 = instance.boilerplate_of_main)
        if now_main == prev_main:
            return
        main = Main_M3.objects.get(main_owner_of_m3 = instance, boilerplate_of_m3 = instance.boilerplate_of_main)
        main.delete()
        if now_main == MAIN_CHOICES[3][0]:
            main = Main_M3(main_owner_of_m3 = instance, boilerplate_of_m3 = instance.boilerplate_of_main)
            main.save()
    except Main_M3.DoesNotExist:
        if now_main == MAIN_CHOICES[3][0]:
            main = Main_M3(main_owner_of_m3 = instance, boilerplate_of_m3 = instance.boilerplate_of_main)
            main.save()


def create_m4(instance, now_main, prev_main):
    try :
        main = Main_M4.objects.get(main_owner_of_m4 = instance, boilerplate_of_m4 = instance.boilerplate_of_main)
        if now_main == prev_main:
            return
        main = Main_M4.objects.get(main_owner_of_m4 = instance, boilerplate_of_m4 = instance.boilerplate_of_main)
        main.delete()
        if now_main == MAIN_CHOICES[4][0]:
            main = Main_M4(main_owner_of_m4 = instance, boilerplate_of_m4 = instance.boilerplate_of_main)
            main.save()
    except Main_M4.DoesNotExist:
        if now_main == MAIN_CHOICES[4][0]:
            main = Main_M4(main_owner_of_m4 = instance, boilerplate_of_m4 = instance.boilerplate_of_main)
            main.save()


def create_m5(instance, now_main, prev_main):
    try :
        main = Main_M5.objects.get(main_owner_of_m5 = instance, boilerplate_of_m5 = instance.boilerplate_of_main)
        if now_main == prev_main:
            return
        main = Main_M5.objects.get(main_owner_of_m5 = instance, boilerplate_of_m5 = instance.boilerplate_of_main)
        main.delete()
        if now_main == MAIN_CHOICES[5][0]:
            main = Main_M5(main_owner_of_m5 = instance, boilerplate_of_m5 = instance.boilerplate_of_main)
            main.save()
    except Main_M5.DoesNotExist:
        if now_main == MAIN_CHOICES[5][0]:
            main = Main_M5(main_owner_of_m5 = instance, boilerplate_of_m5 = instance.boilerplate_of_main)
            main.save()


def create_m6(instance, now_main, prev_main):
    try :
        main = Main_M6.objects.get(main_owner_of_m6 = instance, boilerplate_of_m6 = instance.boilerplate_of_main)
        if now_main == prev_main:
            return
        main = Main_M6.objects.get(main_owner_of_m6 = instance, boilerplate_of_m6 = instance.boilerplate_of_main)
        main.delete()
        if now_main == MAIN_CHOICES[6][0]:
            main = Main_M6(main_owner_of_m6 = instance, boilerplate_of_m6 = instance.boilerplate_of_main)
            main.save()
    except Main_M6.DoesNotExist:
        if now_main == MAIN_CHOICES[6][0]:
            main = Main_M6(main_owner_of_m6 = instance, boilerplate_of_m6 = instance.boilerplate_of_main)
            main.save()


def create_m7(instance, now_main, prev_main):
    try :
        main = Main_M7.objects.get(main_owner_of_m7 = instance, boilerplate_of_m7 = instance.boilerplate_of_main)
        if now_main == prev_main:
            return
        main = Main_M7.objects.get(main_owner_of_m7 = instance, boilerplate_of_m7 = instance.boilerplate_of_main)
        main.delete()
        if now_main == MAIN_CHOICES[7][0]:
            main = Main_M7(main_owner_of_m7 = instance, boilerplate_of_m7 = instance.boilerplate_of_main)
            main.save()
    except Main_M7.DoesNotExist:
        if now_main == MAIN_CHOICES[7][0]:
            main = Main_M7(main_owner_of_m7 = instance, boilerplate_of_m7 = instance.boilerplate_of_main)
            main.save()


def create_m8(instance, now_main, prev_main):
    try :
        main = Main_M8.objects.get(main_owner_of_m8 = instance, boilerplate_of_m8 = instance.boilerplate_of_main)
        if now_main == prev_main:
            return
        main = Main_M8.objects.get(main_owner_of_m8 = instance, boilerplate_of_m8 = instance.boilerplate_of_main)
        main.delete()
        if now_main == MAIN_CHOICES[8][0]:
            main = Main_M8(main_owner_of_m8 = instance, boilerplate_of_m8 = instance.boilerplate_of_main)
            main.save()
    except Main_M8.DoesNotExist:
        if now_main == MAIN_CHOICES[8][0]:
            main = Main_M8(main_owner_of_m8 = instance, boilerplate_of_m8 = instance.boilerplate_of_main)
            main.save()


def create_m9(instance, now_main, prev_main):
    try :
        main = Main_M9.objects.get(main_owner_of_m9 = instance, boilerplate_of_m9 = instance.boilerplate_of_main)
        if now_main == prev_main:
            return
        main = Main_M9.objects.get(main_owner_of_m9 = instance, boilerplate_of_m9 = instance.boilerplate_of_main)
        main.delete()
        if now_main == MAIN_CHOICES[9][0]:
            main = Main_M9(main_owner_of_m9 = instance, boilerplate_of_m9 = instance.boilerplate_of_main)
            main.save()
    except Main_M9.DoesNotExist:
        if now_main == MAIN_CHOICES[9][0]:
            main = Main_M9(main_owner_of_m9 = instance, boilerplate_of_m9 = instance.boilerplate_of_main)
            main.save()


def create_m10(instance, now_main, prev_main):
    try :
        main = Main_M10.objects.get(main_owner_of_m10 = instance, boilerplate_of_m10 = instance.boilerplate_of_main)
        if now_main == prev_main:
            return
        main = Main_M10.objects.get(main_owner_of_m10 = instance, boilerplate_of_m10 = instance.boilerplate_of_main)
        main.delete()
        if now_main == MAIN_CHOICES[10][0]:
            main = Main_M10(main_owner_of_m10 = instance, boilerplate_of_m10 = instance.boilerplate_of_main)
            main.save()
    except Main_M10.DoesNotExist:
        if now_main == MAIN_CHOICES[10][0]:
            main = Main_M10(main_owner_of_m10 = instance, boilerplate_of_m10 = instance.boilerplate_of_main)
            main.save()


def create_m11(instance, now_main, prev_main):
    try :
        main = Main_M11.objects.get(main_owner_of_m11 = instance, boilerplate_of_m11 = instance.boilerplate_of_main)
        if now_main == prev_main:
            return
        main = Main_M11.objects.get(main_owner_of_m11 = instance, boilerplate_of_m11 = instance.boilerplate_of_main)
        main.delete()
        if now_main == MAIN_CHOICES[11][0]:
            main = Main_M11(main_owner_of_m11 = instance, boilerplate_of_m11 = instance.boilerplate_of_main)
            main.save()
    except Main_M11.DoesNotExist:
        if now_main == MAIN_CHOICES[11][0]:
            main = Main_M11(main_owner_of_m11 = instance, boilerplate_of_m11 = instance.boilerplate_of_main)
            main.save()


def create_m12(instance, now_main, prev_main):
    try :
        main = Main_M12.objects.get(main_owner_of_m12 = instance, boilerplate_of_m12 = instance.boilerplate_of_main)
        if now_main == prev_main:
            return
        main = Main_M12.objects.get(main_owner_of_m12 = instance, boilerplate_of_m12 = instance.boilerplate_of_main)
        main.delete()
        if now_main == MAIN_CHOICES[12][0]:
            main = Main_M12(main_owner_of_m12 = instance, boilerplate_of_m12 = instance.boilerplate_of_main)
            main.save()
    except Main_M12.DoesNotExist:
        if now_main == MAIN_CHOICES[12][0]:
            main = Main_M12(main_owner_of_m12 = instance, boilerplate_of_m12 = instance.boilerplate_of_main)
            main.save()


def create_m13(instance, now_main, prev_main):
    try :
        main = Main_M13.objects.get(main_owner_of_m13 = instance, boilerplate_of_m13 = instance.boilerplate_of_main)
        if now_main == prev_main:
            return
        main = Main_M13.objects.get(main_owner_of_m13 = instance, boilerplate_of_m13 = instance.boilerplate_of_main)
        main.delete()
        if now_main == MAIN_CHOICES[13][0]:
            main = Main_M13(main_owner_of_m13 = instance, boilerplate_of_m13 = instance.boilerplate_of_main)
            main.save()
    except Main_M13.DoesNotExist:
        if now_main == MAIN_CHOICES[13][0]:
            main = Main_M13(main_owner_of_m13 = instance, boilerplate_of_m13 = instance.boilerplate_of_main)
            main.save()


def create_m14(instance, now_main, prev_main):
    try :
        main = Main_M14.objects.get(main_owner_of_m14 = instance, boilerplate_of_m14 = instance.boilerplate_of_main)
        if now_main == prev_main:
            return
        main = Main_M14.objects.get(main_owner_of_m14 = instance, boilerplate_of_m14 = instance.boilerplate_of_main)
        main.delete()
        if now_main == MAIN_CHOICES[14][0]:
            main = Main_M14(main_owner_of_m14 = instance, boilerplate_of_m14 = instance.boilerplate_of_main)
            main.save()
    except Main_M14.DoesNotExist:
        if now_main == MAIN_CHOICES[14][0]:
            main = Main_M14(main_owner_of_m14 = instance, boilerplate_of_m14 = instance.boilerplate_of_main)
            main.save()


def create_m15(instance, now_main, prev_main):
    try :
        main = Main_M15.objects.get(main_owner_of_m15 = instance, boilerplate_of_m15 = instance.boilerplate_of_main)
        if now_main == prev_main:
            return
        main = Main_M15.objects.get(main_owner_of_m15 = instance, boilerplate_of_m15 = instance.boilerplate_of_main)
        main.delete()
        if now_main == MAIN_CHOICES[15][0]:
            main = Main_M15(main_owner_of_m15 = instance, boilerplate_of_m15 = instance.boilerplate_of_main)
            main.save()
    except Main_M15.DoesNotExist:
        if now_main == MAIN_CHOICES[15][0]:
            main = Main_M15(main_owner_of_m15 = instance, boilerplate_of_m15 = instance.boilerplate_of_main)
            main.save()


def create_m16(instance, now_main, prev_main):
    try :
        main = Main_M16.objects.get(main_owner_of_m16 = instance, boilerplate_of_m16 = instance.boilerplate_of_main)
        if now_main == prev_main:
            return
        main = Main_M16.objects.get(main_owner_of_m16 = instance, boilerplate_of_m16 = instance.boilerplate_of_main)
        main.delete()
        if now_main == MAIN_CHOICES[16][0]:
            main = Main_M16(main_owner_of_m16 = instance, boilerplate_of_m16 = instance.boilerplate_of_main)
            main.save()
    except Main_M16.DoesNotExist:
        if now_main == MAIN_CHOICES[16][0]:
            main = Main_M16(main_owner_of_m16 = instance, boilerplate_of_m16 = instance.boilerplate_of_main)
            main.save()


class Main(models.Model):
    boilerplate_of_main = models.ForeignKey('reqtool.Boilerplate', related_name='main_boilerplate', on_delete=models.CASCADE)
    main_owner = models.ForeignKey(User, related_name='main_owner', on_delete=models.CASCADE)
    main_choices = models.CharField(choices=MAIN_CHOICES, max_length=100)

    

    def __init__(self, *args, **kwargs):
        super(Main, self).__init__(*args, **kwargs)
        self.__important_fields = ['main_choices']
        for field in self.__important_fields:
            setattr(self, '__original_%s' % field, getattr(self, field))
        

    def save(self, **kwargs):
        """Create P1, P2 or P3 """
        prev_main_list = []
        super(Main, self).save(**kwargs)
        for field in self.__important_fields:
            orig = '__original_%s' % field
            previous_main = getattr(self, orig)
            prev_main_list.append(previous_main)        
        '''Create [M1 - M16]'''
        create_m1( self, self.main_choices, prev_main_list[0])
        create_m2( self, self.main_choices, prev_main_list[0])
        create_m3( self, self.main_choices, prev_main_list[0])
        create_m4( self, self.main_choices, prev_main_list[0])
        create_m5( self, self.main_choices, prev_main_list[0])
        create_m6( self, self.main_choices, prev_main_list[0])
        create_m7( self, self.main_choices, prev_main_list[0])
        create_m8( self, self.main_choices, prev_main_list[0])
        create_m9( self, self.main_choices, prev_main_list[0])
        create_m10( self, self.main_choices, prev_main_list[0])
        create_m11( self, self.main_choices, prev_main_list[0])
        create_m12( self, self.main_choices, prev_main_list[0])
        create_m13( self, self.main_choices, prev_main_list[0])
        create_m14( self, self.main_choices, prev_main_list[0])
        create_m15( self, self.main_choices, prev_main_list[0])
        create_m16( self, self.main_choices, prev_main_list[0])



    class Meta:
        ordering = ['boilerplate_of_main']


class Main_M1(models.Model):
    boilerplate_of_m1 = models.ForeignKey('reqtool.Boilerplate', related_name='main_boilerplate_of_M1', on_delete=models.CASCADE)
    main_owner_of_m1 = models.ForeignKey(Main, related_name='main_owner_of_M1', on_delete=models.CASCADE)
    sys_fun_m1 = models.CharField(choices=SYSTEM_OR_FUNCTION_CHOICES, max_length=100)
    shall_m1 = models.CharField(choices=SHALL_CHOICES, max_length=100)
    quantifier_m1 = models.CharField(choices=QUANTIFIER_CHOICES, max_length=100)
    numerical_m1 = models.IntegerField(default=0, blank=True) 
    mumerical_units_m1 = models.CharField(choices=NUMBER_UNITS_CHOICES, max_length=100)
    item_m1 = models.CharField(choices=ITEM_CHOICES, max_length=100)
    statevalue_m1 = models.CharField(choices=STATE_VALUE_CHOICES, max_length=100)
        

    class Meta:
        ordering = ['boilerplate_of_m1']


class Main_M2(models.Model):
    boilerplate_of_m2 = models.ForeignKey('reqtool.Boilerplate', related_name='main_boilerplate_of_M2', on_delete=models.CASCADE)
    main_owner_of_m2 = models.ForeignKey(Main, related_name='main_owner_of_M2', on_delete=models.CASCADE)
    sys_fun_m2 = models.CharField(choices=SYSTEM_OR_FUNCTION_CHOICES, max_length=100)
    shall_m2 = models.CharField(choices=SHALL_CHOICES, max_length=100)
    state_m2 = models.CharField(default = ' ', max_length=100)
    statevalue_m2 = models.CharField(choices=STATE_VALUE_CHOICES, max_length=100)
        

    class Meta:
        ordering = ['boilerplate_of_m2']


class Main_M3(models.Model):
    boilerplate_of_m3 = models.ForeignKey('reqtool.Boilerplate', related_name='main_boilerplate_of_M3', on_delete=models.CASCADE)
    main_owner_of_m3 = models.ForeignKey(Main, related_name='main_owner_of_M3', on_delete=models.CASCADE)
    sys_fun_m3 = models.CharField(choices=SYSTEM_OR_FUNCTION_CHOICES, max_length=100)
    shall_m3 = models.CharField(choices=SHALL_CHOICES, max_length=100)
    quantifier_m3 = models.CharField(choices=QUANTIFIER_CHOICES, max_length=100)
    numerical_m3 = models.IntegerField(default=0, blank=True) 
    mumerical_units_m3 = models.CharField(choices=NUMBER_UNITS_CHOICES, max_length=100)
    item_m3 = models.CharField(choices=ITEM_CHOICES, max_length=100)
   

    class Meta:
        ordering = ['boilerplate_of_m3']


class Main_M4(models.Model):
    boilerplate_of_m4 = models.ForeignKey('reqtool.Boilerplate', related_name='main_boilerplate_of_M4', on_delete=models.CASCADE)
    main_owner_of_m4 = models.ForeignKey(Main, related_name='main_owner_of_M4', on_delete=models.CASCADE)
    sys_fun_m4 = models.CharField(choices=SYSTEM_OR_FUNCTION_CHOICES, max_length=100)
    shall_m4 = models.CharField(choices=SHALL_CHOICES, max_length=100)
    quantifier_m4 = models.CharField(choices=QUANTIFIER_CHOICES, max_length=100)
    numerical_m4 = models.IntegerField(default=0, blank=True) 
    mumerical_units_m4 = models.CharField(choices=NUMBER_UNITS_CHOICES, max_length=100)
    item_m4 = models.CharField(choices=ITEM_CHOICES, max_length=100)
   

    class Meta:
        ordering = ['boilerplate_of_m4']


class Main_M5(models.Model):
    boilerplate_of_m5 = models.ForeignKey('reqtool.Boilerplate', related_name='main_boilerplate_of_M5', on_delete=models.CASCADE)
    main_owner_of_m5 = models.ForeignKey(Main, related_name='main_owner_of_M5', on_delete=models.CASCADE)
    sys_fun_m5 = models.CharField(choices=SYSTEM_OR_FUNCTION_CHOICES, max_length=100)
    shall_m5 = models.CharField(choices=SHALL_CHOICES, max_length=100)
    flow_m5 = models.CharField(choices=FLOW_CHOICES, max_length=100)
   

    class Meta:
        ordering = ['boilerplate_of_m5']


class Main_M6(models.Model):
    boilerplate_of_m6 = models.ForeignKey('reqtool.Boilerplate', related_name='main_boilerplate_of_M6', on_delete=models.CASCADE)
    main_owner_of_m6 = models.ForeignKey(Main, related_name='main_owner_of_M6', on_delete=models.CASCADE)
    sys_fun_m6 = models.CharField(choices=SYSTEM_OR_FUNCTION_CHOICES, max_length=100)
    shall_m6 = models.CharField(choices=SHALL_CHOICES, max_length=100)
    flow_m6 = models.CharField(choices=FLOW_CHOICES, max_length=100)
   

    class Meta:
        ordering = ['boilerplate_of_m6']


class Main_M7(models.Model):
    boilerplate_of_m7 = models.ForeignKey('reqtool.Boilerplate', related_name='main_boilerplate_of_M7', on_delete=models.CASCADE)
    main_owner_of_m7 = models.ForeignKey(Main, related_name='main_owner_of_M7', on_delete=models.CASCADE)
    system_m7 = models.CharField(choices=SYSTEM_CHOICES, max_length=100)
    shall_m7 = models.CharField(choices=SHALL_CHOICES, max_length=100)
    function_m7 = models.CharField(choices=FUNCTION_CHOICES, max_length=100)

    class Meta:
        ordering = ['boilerplate_of_m7']


class Main_M8(models.Model):
    boilerplate_of_m8 = models.ForeignKey('reqtool.Boilerplate', related_name='main_boilerplate_of_M8', on_delete=models.CASCADE)
    main_owner_of_m8 = models.ForeignKey(Main, related_name='main_owner_of_M8', on_delete=models.CASCADE)
    function_1_m8 = models.CharField(choices=FUNCTION_CHOICES, max_length=100)
    shall_m8 = models.CharField(choices=SHALL_CHOICES, max_length=100)
    function_2_m8 = models.CharField(choices=FUNCTION_CHOICES, max_length=100)

    class Meta:
        ordering = ['boilerplate_of_m8']


class Main_M9(models.Model):
    boilerplate_of_m9 = models.ForeignKey('reqtool.Boilerplate', related_name='main_boilerplate_of_M9', on_delete=models.CASCADE)
    main_owner_of_m9 = models.ForeignKey(Main, related_name='main_owner_of_M9', on_delete=models.CASCADE)
    system_1_m9 = models.CharField(choices=SYSTEM_CHOICES, max_length=100)
    shall_m9 = models.CharField(choices=SHALL_CHOICES, max_length=100)
    interface_m9 = models.CharField(choices=INTERFACE_CHOICES, max_length=100)
    system_2_m9 = models.CharField(choices=SYSTEM_CHOICES, max_length=100)

    class Meta:
        ordering = ['boilerplate_of_m9']


class Main_M10(models.Model):
    boilerplate_of_m10 = models.ForeignKey('reqtool.Boilerplate', related_name='main_boilerplate_of_M10', on_delete=models.CASCADE)
    main_owner_of_m10 = models.ForeignKey(Main, related_name='main_owner_of_M10', on_delete=models.CASCADE)
    sys_inter_m10 = models.CharField(choices=SYSTEM_INTERFACE_CHOICES, max_length=100)
    shall_m10 = models.CharField(choices=SHALL_CHOICES, max_length=100)
    flow_item_m9 = models.CharField(choices=FLOW_ITEM_CHOICES, max_length=100)

    class Meta:
        ordering = ['boilerplate_of_m10']

#need to ask about [as dened in <text>]
class Main_M11(models.Model):
    boilerplate_of_m11 = models.ForeignKey('reqtool.Boilerplate', related_name='main_boilerplate_of_M11', on_delete=models.CASCADE)
    main_owner_of_m11 = models.ForeignKey(Main, related_name='main_owner_of_M11', on_delete=models.CASCADE)
    system_1_m11 = models.CharField(choices=SYSTEM_CHOICES, max_length=100)
    shall_m11 = models.CharField(choices=SHALL_CHOICES, max_length=100)
    system_2_m11 = models.CharField(choices=SYSTEM_CHOICES, max_length=100)
    connection_m11 = models.CharField(choices=CONNECTION_CHOICES, max_length=100)

    class Meta:
        ordering = ['boilerplate_of_m11']

#need to ask about [with values stateSet]
class Main_M12(models.Model):
    boilerplate_of_m12 = models.ForeignKey('reqtool.Boilerplate', related_name='main_boilerplate_of_M12', on_delete=models.CASCADE)
    main_owner_of_m12 = models.ForeignKey(Main, related_name='main_owner_of_M12', on_delete=models.CASCADE)
    system_m12 = models.CharField(choices=SYSTEM_CHOICES, max_length=100)
    shall_m12 = models.CharField(choices=SHALL_CHOICES, max_length=100)
    state_m12 = models.CharField(choices=STATE_CHOICES, max_length=100)

    class Meta:
        ordering = ['boilerplate_of_m12']

#need to ask about [with values stateSet]
class Main_M13(models.Model):
    boilerplate_of_m13 = models.ForeignKey('reqtool.Boilerplate', related_name='main_boilerplate_of_M13', on_delete=models.CASCADE)
    main_owner_of_m13 = models.ForeignKey(Main, related_name='main_owner_of_M13', on_delete=models.CASCADE)
    system_m13 = models.CharField(choices=SYSTEM_CHOICES, max_length=100)
    state_1_m13 = models.CharField(choices=STATE_CHOICES, max_length=100)
    statevalue_m13 = models.CharField(choices=STATE_VALUE_CHOICES, max_length=100)
    shall_m13 = models.CharField(choices=SHALL_CHOICES, max_length=100)
    state_2_m13 = models.CharField(choices=STATE_CHOICES, max_length=100)

    class Meta:
        ordering = ['boilerplate_of_m13']


class Main_M14(models.Model):
    boilerplate_of_m14 = models.ForeignKey('reqtool.Boilerplate', related_name='main_boilerplate_of_M14', on_delete=models.CASCADE)
    main_owner_of_m14 = models.ForeignKey(Main, related_name='main_owner_of_M14', on_delete=models.CASCADE)
    system_m14 = models.CharField(choices=SYSTEM_CHOICES, max_length=100)
    item_m14 = models.CharField(choices=ITEM_CHOICES, max_length=100)
    shall_m14 = models.CharField(choices=SHALL_CHOICES, max_length=100)
    #needs stateSET

    class Meta:
        ordering = ['boilerplate_of_m14']


class Main_M15(models.Model):
    boilerplate_of_m15 = models.ForeignKey('reqtool.Boilerplate', related_name='main_boilerplate_of_M15', on_delete=models.CASCADE)
    main_owner_of_m15 = models.ForeignKey(Main, related_name='main_owner_of_M15', on_delete=models.CASCADE)
    system_m15 = models.CharField(choices=SYSTEM_CHOICES, max_length=100)
    item_1_m15 = models.CharField(choices=ITEM_CHOICES, max_length=100)
    shall_m15 = models.CharField(choices=SHALL_CHOICES, max_length=100)
    item_2_m15 = models.CharField(choices=ITEM_CHOICES, max_length=100)

    class Meta:
        ordering = ['boilerplate_of_m15']


class Main_M16(models.Model):
    boilerplate_of_m16 = models.ForeignKey('reqtool.Boilerplate', related_name='main_boilerplate_of_M16', on_delete=models.CASCADE)
    main_owner_of_m16 = models.ForeignKey(Main, related_name='main_owner_of_M16', on_delete=models.CASCADE)
    system_1_m16 = models.CharField(choices=SYSTEM_CHOICES, max_length=100)
    shall_m16 = models.CharField(choices=SHALL_CHOICES, max_length=100)
    system_2_m16 = models.CharField(choices=SYSTEM_CHOICES, max_length=100)

    class Meta:
        ordering = ['boilerplate_of_m16']

