from django.db import models
#from reqman.apps.common.models import CoreModel
from reqman.apps.account.models import User
from reqman.apps.reqtool.models.main_req import create_main
from reqman.apps.reqtool.models.suffix_req import create_suffix

from pygments.formatters.html import HtmlFormatter
from pygments import highlight

SIMPLE_PREFIX_CHOICES = (   ("",""),  
                            ("P1", "If/Unless <logical expression>"), 
                            ("P2", "As soon as <occuring functionality>"), 
                            ("P3", "As long as <occuring functionality>")
                        )

LOGIC_CONNECTIVE = (    ("",""), 
                        ("or","OR"), 
                        ("and", "AND"), 
                        ("xor", "XOR")
                    )

LOGICAL_EXPRESSION = (  ("",""), 
                        ("state value constraint", "state value constraint"), 
                        ("occuring functionality", "occuring functionality")
                    )

def create_prefix(instance):
    '''Create the Prefix Model for the Boilerplate'''
    try :
        prefix = Prefix.objects.get(boilerplate = instance, prefix_owner = instance.owner)
        if instance.has_prefix:
            return
        prefix = Prefix.objects.get(boilerplate = instance, prefix_owner = instance.owner)
        prefix.delete()
    except Prefix.DoesNotExist:
        if instance.has_prefix:
            prefix = Prefix(boilerplate = instance, prefix_owner = instance.owner)
            prefix.save()
    

def create_p1(instance, now_prefix, prev_prefix, series_of_prefix):
    try :
        prefix = LogicalExpression.objects.get(prefix = instance, prefix_boilerplate = instance.boilerplate, prefix_series = series_of_prefix)
        if now_prefix == prev_prefix:
            return
        prefix = LogicalExpression.objects.get(prefix = instance, prefix_boilerplate = instance.boilerplate, prefix_series = series_of_prefix )
        prefix.delete()
        if now_prefix == SIMPLE_PREFIX_CHOICES[1][0]:
            prefix = LogicalExpression(prefix = instance, prefix_boilerplate = instance.boilerplate, prefix_series = series_of_prefix)
            prefix.save()
    except LogicalExpression.DoesNotExist:
        if now_prefix == SIMPLE_PREFIX_CHOICES[1][0]:
            prefix = LogicalExpression(prefix = instance, prefix_boilerplate = instance.boilerplate, prefix_series = series_of_prefix)
            prefix.save()


def create_p2(instance, now_prefix, prev_prefix, series_of_prefix):
    try :
        prefix = OccuringFunctionalityP2.objects.get(prefix_of_occuring_functionality_p2 = instance, prefix_boilerplate_of_occuring_functionality_p2 = instance.boilerplate, prefix_series_of_occuring_functionality_p2 = series_of_prefix)
        if now_prefix == prev_prefix:
            return
        prefix = OccuringFunctionalityP2.objects.get(prefix_of_occuring_functionality_p2 = instance, prefix_boilerplate_of_occuring_functionality_p2 = instance.boilerplate, prefix_series_of_occuring_functionality_p2 = series_of_prefix )
        prefix.delete()
        if now_prefix == SIMPLE_PREFIX_CHOICES[2][0]:
            prefix = OccuringFunctionalityP2(prefix_of_occuring_functionality_p2 = instance, prefix_boilerplate_of_occuring_functionality_p2 = instance.boilerplate, prefix_series_of_occuring_functionality_p2 = series_of_prefix)
            prefix.save()
    except OccuringFunctionalityP2.DoesNotExist:
        if now_prefix == SIMPLE_PREFIX_CHOICES[2][0]:
            prefix = OccuringFunctionalityP2(prefix_of_occuring_functionality_p2 = instance, prefix_boilerplate_of_occuring_functionality_p2 = instance.boilerplate, prefix_series_of_occuring_functionality_p2 = series_of_prefix)
            prefix.save()


def create_p3(instance, now_prefix, prev_prefix, series_of_prefix):
    try :
        prefix = OccuringFunctionalityP3.objects.get(prefix_of_occuring_functionality_p3 = instance, prefix_boilerplate_of_occuring_functionality_p3 = instance.boilerplate, prefix_series_of_occuring_functionality_p3 = series_of_prefix)
        if now_prefix == prev_prefix:
            return
        prefix = OccuringFunctionalityP3.objects.get(prefix_of_occuring_functionality_p3 = instance, prefix_boilerplate_of_occuring_functionality_p3 = instance.boilerplate, prefix_series_of_occuring_functionality_p3 = series_of_prefix )
        prefix.delete()
        if now_prefix == SIMPLE_PREFIX_CHOICES[3][0]:
            prefix = OccuringFunctionalityP3(prefix_of_occuring_functionality_p3 = instance, prefix_boilerplate_of_occuring_functionality_p3 = instance.boilerplate, prefix_series_of_occuring_functionality_p3 = series_of_prefix)
            prefix.save()
    except OccuringFunctionalityP3.DoesNotExist:
        if now_prefix == SIMPLE_PREFIX_CHOICES[3][0]:
            prefix = OccuringFunctionalityP3(prefix_of_occuring_functionality_p3 = instance, prefix_boilerplate_of_occuring_functionality_p3 = instance.boilerplate, prefix_series_of_occuring_functionality_p3 = series_of_prefix)
            prefix.save()


def create_state_value_constraint_from_logical_expression(instance, logical_expression, prev_logical_expression, series_of_log_expr):
    try :
        log_expr = StateValueConstraint.objects.get(logical_expression = instance, prefix_of_logical_expression = instance.prefix, log_expr_series = series_of_log_expr)
        if logical_expression == prev_logical_expression:
            return
        log_expr = StateValueConstraint.objects.get(logical_expression = instance, prefix_of_logical_expression = instance.prefix, log_expr_series = series_of_log_expr )
        log_expr.delete()
        if logical_expression == LOGICAL_EXPRESSION[1][0]:
            log_expr = StateValueConstraint(logical_expression = instance, prefix_of_logical_expression = instance.prefix, log_expr_series = series_of_log_expr)
            log_expr.save()
    except StateValueConstraint.DoesNotExist:
        if logical_expression == LOGICAL_EXPRESSION[1][0]:
            log_expr = StateValueConstraint(logical_expression = instance, prefix_of_logical_expression = instance.prefix, log_expr_series = series_of_log_expr)
            log_expr.save()


def create_state_occuring_functionality_from_logical_expression(instance, logical_expression, prev_logical_expression, series_of_log_expr):
    try :
        log_expr = OccuringFunctionalityForLogicalExpression.objects.get(logical_expression_of_occuring = instance, prefix_of_occuring_for_logical_expression = instance.prefix, log_expres_series = series_of_log_expr)
        if logical_expression == prev_logical_expression:
            return
        log_expr = OccuringFunctionalityForLogicalExpression.objects.get(logical_expression_of_occuring = instance, prefix_of_occuring_for_logical_expression = instance.prefix, log_expres_series = series_of_log_expr )
        log_expr.delete()
        if logical_expression == LOGICAL_EXPRESSION[2][0]:
            log_expr = OccuringFunctionalityForLogicalExpression(logical_expression_of_occuring = instance, prefix_of_occuring_for_logical_expression = instance.prefix, log_expres_series = series_of_log_expr)
            log_expr.save()
    except OccuringFunctionalityForLogicalExpression.DoesNotExist:
        if logical_expression == LOGICAL_EXPRESSION[2][0]:
            log_expr = OccuringFunctionalityForLogicalExpression(logical_expression_of_occuring = instance, prefix_of_occuring_for_logical_expression = instance.prefix, log_expres_series = series_of_log_expr)
            log_expr.save()



class Boilerplate(models.Model):
    owner = models.ForeignKey(User, related_name='boilerplate', on_delete=models.CASCADE)    
    created = models.DateTimeField(auto_now_add=True)
    has_prefix = models.BooleanField(default=False)
    has_main = models.BooleanField(default=False)
    has_suffix = models.BooleanField(default=False)
     

    def save(self, **kwargs):
        """Create prefix, main or suffix """
        super(Boilerplate, self).save(**kwargs)
        create_prefix(self)
        create_main(self)
        create_suffix(self)
        
        
    class Meta:
        ordering = ['created']

class Prefix(models.Model):
    #boilerplate = models.OneToOneField(Boilerplate, related_name='prefix', on_delete=models.CASCADE, primary_key=True)
    boilerplate = models.ForeignKey(Boilerplate, related_name='prefix', on_delete=models.CASCADE)
    prefix_owner = models.ForeignKey(User, related_name='prefix_owner', on_delete=models.CASCADE)    
    simple_prefix_1 = models.CharField(choices=SIMPLE_PREFIX_CHOICES, max_length=100)
    logic_connective_1 = models.CharField(choices=LOGIC_CONNECTIVE, default='', max_length=100, null=True)
    simple_prefix_2 =  models.CharField(choices=SIMPLE_PREFIX_CHOICES, default='', max_length=100, null=True)
    logic_connective_2 = models.CharField(choices=LOGIC_CONNECTIVE, default='', max_length=100, null=True)
    simple_prefix_3 =  models.CharField(choices=SIMPLE_PREFIX_CHOICES, default='', max_length=100, null=True)
    

    def __init__(self, *args, **kwargs):
        super(Prefix, self).__init__(*args, **kwargs)
        self.__important_fields = ['simple_prefix_1', 'simple_prefix_2', 'simple_prefix_3']
        for field in self.__important_fields:
            setattr(self, '__original_%s' % field, getattr(self, field))
        

    def save(self, **kwargs):
        """Create P1, P2 or P3 """
        prev_prefix_list = []
        super(Prefix, self).save(**kwargs)
        for field in self.__important_fields:
            orig = '__original_%s' % field
            previous_simple_prefix = getattr(self, orig)
            prev_prefix_list.append(previous_simple_prefix)        
        '''Create P1 '''
        create_p1( self, self.simple_prefix_1, prev_prefix_list[0], '1')
        create_p1( self, self.simple_prefix_2, prev_prefix_list[1], '2')
        create_p1( self, self.simple_prefix_3, prev_prefix_list[2], '3')
        '''Create P2 '''
        create_p2( self, self.simple_prefix_1, prev_prefix_list[0], '1')
        create_p2( self, self.simple_prefix_2, prev_prefix_list[1], '2')
        create_p2( self, self.simple_prefix_3, prev_prefix_list[2], '3')
        '''Create P3 '''
        create_p3( self, self.simple_prefix_1, prev_prefix_list[0], '1')
        create_p3( self, self.simple_prefix_2, prev_prefix_list[1], '2')
        create_p3( self, self.simple_prefix_3, prev_prefix_list[2], '3')
           

    class Meta:
        ordering = ['boilerplate']


class LogicalExpression(models.Model):
    prefix = models.ForeignKey(Prefix, related_name='log_expr', on_delete=models.CASCADE)
    prefix_boilerplate = models.ForeignKey(Boilerplate, related_name='logic_expr', on_delete=models.CASCADE)
    prefix_series = models.TextField(default='0')
    logical_expression_1 =  models.CharField(choices=LOGICAL_EXPRESSION , max_length=100)
    logic_connective_1 = models.CharField(choices=LOGIC_CONNECTIVE , default='', max_length=100, null=True)
    logical_expression_2 =  models.CharField(choices=LOGICAL_EXPRESSION , default='', max_length=100, null=True)
    logic_connective_2 = models.CharField(choices=LOGIC_CONNECTIVE , default='', max_length=100, null=True)
    logical_expression_3 =  models.CharField(choices=LOGICAL_EXPRESSION , default='', max_length=100, null=True)


    def __init__(self, *args, **kwargs):
        super(LogicalExpression, self).__init__(*args, **kwargs)
        self.__important_fields = ['logical_expression_1', 'logical_expression_2', 'logical_expression_3', 'prefix_boilerplate']
        for field in self.__important_fields:
            setattr(self, '__original_%s' % field, getattr(self, field))
    

    def save(self, **kwargs):
        """Create State Value Constraint or Occuring Functionality"""
        prev_log_expr_list = []
        super(LogicalExpression, self).save(**kwargs)
        for field in self.__important_fields:
            orig = '__original_%s' % field
            previous_log_expr = getattr(self, orig)
            prev_log_expr_list.append(previous_log_expr)
        '''Create a state value constraint '''
        create_state_value_constraint_from_logical_expression( self, self.logical_expression_1, prev_log_expr_list[0], '1')
        create_state_value_constraint_from_logical_expression( self, self.logical_expression_2, prev_log_expr_list[1], '2')
        create_state_value_constraint_from_logical_expression( self, self.logical_expression_3, prev_log_expr_list[2], '3')
        '''Create a occuring functionality '''
        create_state_occuring_functionality_from_logical_expression( self, self.logical_expression_1, prev_log_expr_list[0], '1')
        create_state_occuring_functionality_from_logical_expression( self, self.logical_expression_2, prev_log_expr_list[1], '2')
        create_state_occuring_functionality_from_logical_expression( self, self.logical_expression_3, prev_log_expr_list[2], '3')

    class Meta:
        ordering = ['prefix_boilerplate']


class OccuringFunctionalityP2(models.Model):
    prefix_of_occuring_functionality_p2 = models.ForeignKey(Prefix, related_name='occuring_functionality', on_delete=models.CASCADE)
    prefix_boilerplate_of_occuring_functionality_p2 = models.ForeignKey(Boilerplate, related_name='occuring_functionality_b', on_delete=models.CASCADE)
    prefix_series_of_occuring_functionality_p2 = models.TextField(default='0')
    system_for_occuring_functionality_p2 = models.TextField(max_length=100, blank=True,)
    perform_for_occuring_functionality_p2 = models.TextField(max_length=100, blank=True,)
    function_for_occuring_functionality_p2 = models.TextField(max_length=100, blank=True,)
    item_for_occuring_functionality_p2 = models.TextField(max_length=100, blank=True,)
    flow_for_occuring_functionality_p2 = models.TextField(max_length=100, blank=True,)

    class Meta:
        ordering = ['prefix_boilerplate_of_occuring_functionality_p2']


class OccuringFunctionalityP3(models.Model):
    prefix_of_occuring_functionality_p3 = models.ForeignKey(Prefix, related_name='occuring_functionality_p3', on_delete=models.CASCADE)
    prefix_boilerplate_of_occuring_functionality_p3 = models.ForeignKey(Boilerplate, related_name='occuring_functionality_b_p3', on_delete=models.CASCADE)
    prefix_series_of_occuring_functionality_p3 = models.TextField(default='0')
    system_for_occuring_functionality_p3 = models.TextField(max_length=100, blank=True,)
    perform_for_occuring_functionality_p3 = models.TextField(max_length=100, blank=True,)
    function_for_occuring_functionality_p3 = models.TextField(max_length=100, blank=True,)
    item_for_occuring_functionality_p3 = models.TextField(max_length=100, blank=True,)
    flow_for_occuring_functionality_p3 = models.TextField(max_length=100, blank=True,)

    class Meta:
        ordering = ['prefix_boilerplate_of_occuring_functionality_p3']


class StateValueConstraint(models.Model):
    prefix_of_logical_expression = models.ForeignKey(Prefix, related_name='prefix_of_log_expr', on_delete=models.CASCADE)
    logical_expression = models.ForeignKey(LogicalExpression, related_name='own_log_expr', on_delete=models.CASCADE)
    log_expr_series = models.TextField(default='0')
    item = models.TextField(max_length=100, blank=True,)
    system_state = models.TextField(max_length=100, blank=True,)
    state_value = models.TextField(max_length=100, blank=True,)


    class Meta:
        ordering = ['logical_expression']


class OccuringFunctionalityForLogicalExpression(models.Model):
    prefix_of_occuring_for_logical_expression = models.ForeignKey(Prefix, related_name='prefix_of_occur_funct', on_delete=models.CASCADE)
    logical_expression_of_occuring = models.ForeignKey(LogicalExpression, related_name='owner_of_log_expr', on_delete=models.CASCADE)
    log_expres_series = models.TextField(default='0')
    #model to be done (send,receives, ingests etc.)
    system_for_logical_expression = models.TextField(max_length=100, blank=True,)
    perform_for_logical_expression = models.TextField(max_length=100, blank=True,)
    function_for_logical_expression = models.TextField(max_length=100, blank=True,)
    item_for_logical_expression = models.TextField(max_length=100, blank=True,)
    flow_for_logical_expression = models.TextField(max_length=100, blank=True,)

    class Meta:
        ordering = ['logical_expression_of_occuring']