from rest_framework.generics import ListCreateAPIView, RetrieveUpdateDestroyAPIView, ListAPIView
from rest_framework import permissions

from django.http import HttpResponse, JsonResponse
from django.views.decorators.csrf import csrf_exempt
from rest_framework.parsers import JSONParser

from reqman.apps.reqtool.models.suffix_req import Suffix, Suffix_S1, Suffix_S2, Suffix_S3, Suffix_S4, Suffix_S5, Suffix_S6

from reqman.apps.reqtool.rest_api.serializers.suffix_req import SuffixSerializer, Suffix_S1Serializer, Suffix_S2Serializer, Suffix_S3Serializer, Suffix_S4Serializer, Suffix_S5Serializer, Suffix_S6Serializer

from reqman.apps.permissions import IsOwnerOrReadOnly
from reqman.apps.reqtool.rest_api.services import fuseki

class SuffixListAPIView(ListAPIView):
    """
    API view to retrieve list of posts or create new
    """
    permission_classes = [permissions.IsAuthenticatedOrReadOnly]
    serializer_class = SuffixSerializer
    #
    queryset = Suffix.objects.all()

    def get_queryset(self):
        group_pk = self.kwargs['boilerplate']
        #print(group_pk)
        queryset_suffix = Suffix.objects.filter(boilerplate_of_suffix = group_pk)
        return queryset_suffix

    #Custom actions when POST
    def perform_create(self, serializer):
        serializer.save()
        #print the data of the post
        #write_fuseki = fuseki.FusekiActions()
        #write_fuseki.write(serializer.data)

class SuffixDetailsAPIView(RetrieveUpdateDestroyAPIView):
    """
    API view to retrieve, update or delete post
    """
    permission_classes = [permissions.IsAuthenticatedOrReadOnly]

    serializer_class = SuffixSerializer
    
    def get_queryset(self):
        group_pk = self.kwargs['boilerplate']
        #print(group_pk)
        queryset_suffix = Suffix.objects.filter(boilerplate_of_suffix = group_pk)
        return queryset_suffix

    #Custom actions when PUT
    def perform_update(self, serializer):
        serializer.save()
        #print(self.new.changed_data)
        #instance = serializer.save()
        #write_fuseki = fuseki.FusekiActions()
        #write_fuseki.write(serializer.data)

    #Custom actions when DELETE
    def perform_destroy(self, instance):
        #print("deleted")
        instance.delete()


class Suffix_S1ListAPIView(ListAPIView):
    """
    API view to retrieve list of posts or create new
    """
    permission_classes = [permissions.IsAuthenticatedOrReadOnly]
    serializer_class = Suffix_S1Serializer

    def get_queryset(self):
        bp = self.kwargs['boilerplate']
        sf = self.kwargs['suffixs']
        #print(group_pk)
        queryset_suffix = Suffix_S1.objects.filter(boilerplate_of_s1 = bp, suffix_owner_of_s1 = sf)
        return queryset_suffix

    #Custom actions when POST
    def perform_create(self, serializer):
        serializer.save()
        #print the data of the post
        #write_fuseki = fuseki.FusekiActions()
        #write_fuseki.write(serializer.data)

class Suffix_S1DetailsAPIView(RetrieveUpdateDestroyAPIView):
    """
    API view to retrieve, update or delete post
    """
    permission_classes = [permissions.IsAuthenticatedOrReadOnly]
    serializer_class = Suffix_S1Serializer

    def get_queryset(self):
        bp = self.kwargs['boilerplate']
        sf = self.kwargs['suffixs']
        #print(group_pk)
        queryset_suffix = Suffix_S1.objects.filter(boilerplate_of_s1 = bp, suffix_owner_of_s1 = sf)
        return queryset_suffix

    #Custom actions when PUT
    def perform_update(self, serializer):
        serializer.save()
        #print(self.new.changed_data)
        #instance = serializer.save()
        #write_fuseki = fuseki.FusekiActions()
        #write_fuseki.write(serializer.data)

    #Custom actions when DELETE
    def perform_destroy(self, instance):
        #print("deleted")
        instance.delete()


class Suffix_S2ListAPIView(ListAPIView):
    """
    API view to retrieve list of posts or create new
    """
    permission_classes = [permissions.IsAuthenticatedOrReadOnly]
    serializer_class = Suffix_S2Serializer

    def get_queryset(self):
        bp = self.kwargs['boilerplate']
        sf = self.kwargs['suffixs']
        #print(group_pk)
        queryset_suffix = Suffix_S2.objects.filter(boilerplate_of_s2 = bp, suffix_owner_of_s2 = sf)
        return queryset_suffix

    #Custom actions when POST
    def perform_create(self, serializer):
        serializer.save()
        #print the data of the post
        #write_fuseki = fuseki.FusekiActions()
        #write_fuseki.write(serializer.data)

class Suffix_S2DetailsAPIView(RetrieveUpdateDestroyAPIView):
    """
    API view to retrieve, update or delete post
    """
    permission_classes = [permissions.IsAuthenticatedOrReadOnly]
    serializer_class = Suffix_S2Serializer

    def get_queryset(self):
        bp = self.kwargs['boilerplate']
        sf = self.kwargs['suffixs']
        #print(group_pk)
        queryset_suffix = Suffix_S2.objects.filter(boilerplate_of_s2 = bp, suffix_owner_of_s2 = sf)
        return queryset_suffix

    #Custom actions when PUT
    def perform_update(self, serializer):
        serializer.save()
        #print(self.new.changed_data)
        #instance = serializer.save()
        #write_fuseki = fuseki.FusekiActions()
        #write_fuseki.write(serializer.data)

    #Custom actions when DELETE
    def perform_destroy(self, instance):
        #print("deleted")
        instance.delete()


class Suffix_S3ListAPIView(ListAPIView):
    """
    API view to retrieve list of posts or create new
    """
    permission_classes = [permissions.IsAuthenticatedOrReadOnly]
    serializer_class = Suffix_S3Serializer

    def get_queryset(self):
        bp = self.kwargs['boilerplate']
        sf = self.kwargs['suffixs']
        #print(group_pk)
        queryset_suffix = Suffix_S3.objects.filter(boilerplate_of_s3 = bp, suffix_owner_of_s3 = sf)
        return queryset_suffix

    #Custom actions when POST
    def perform_create(self, serializer):
        serializer.save()
        #print the data of the post
        #write_fuseki = fuseki.FusekiActions()
        #write_fuseki.write(serializer.data)

class Suffix_S3DetailsAPIView(RetrieveUpdateDestroyAPIView):
    """
    API view to retrieve, update or delete post
    """
    permission_classes = [permissions.IsAuthenticatedOrReadOnly]
    serializer_class = Suffix_S3Serializer

    def get_queryset(self):
        bp = self.kwargs['boilerplate']
        sf = self.kwargs['suffixs']
        #print(group_pk)
        queryset_suffix = Suffix_S3.objects.filter(boilerplate_of_s3 = bp, suffix_owner_of_s3 = sf)
        return queryset_suffix

    #Custom actions when PUT
    def perform_update(self, serializer):
        serializer.save()
        #print(self.new.changed_data)
        #instance = serializer.save()
        #write_fuseki = fuseki.FusekiActions()
        #write_fuseki.write(serializer.data)

    #Custom actions when DELETE
    def perform_destroy(self, instance):
        #print("deleted")
        instance.delete()


class Suffix_S4ListAPIView(ListAPIView):
    """
    API view to retrieve list of posts or create new
    """
    permission_classes = [permissions.IsAuthenticatedOrReadOnly]
    serializer_class = Suffix_S4Serializer

    def get_queryset(self):
        bp = self.kwargs['boilerplate']
        sf = self.kwargs['suffixs']
        #print(group_pk)
        queryset_suffix = Suffix_S4.objects.filter(boilerplate_of_s4 = bp, suffix_owner_of_s4 = sf)
        return queryset_suffix

    #Custom actions when POST
    def perform_create(self, serializer):
        serializer.save()
        #print the data of the post
        #write_fuseki = fuseki.FusekiActions()
        #write_fuseki.write(serializer.data)

class Suffix_S4DetailsAPIView(RetrieveUpdateDestroyAPIView):
    """
    API view to retrieve, update or delete post
    """
    permission_classes = [permissions.IsAuthenticatedOrReadOnly]
    serializer_class = Suffix_S4Serializer

    def get_queryset(self):
        bp = self.kwargs['boilerplate']
        sf = self.kwargs['suffixs']
        #print(group_pk)
        queryset_suffix = Suffix_S4.objects.filter(boilerplate_of_s4 = bp, suffix_owner_of_s4 = sf)
        return queryset_suffix

    #Custom actions when PUT
    def perform_update(self, serializer):
        serializer.save()
        #print(self.new.changed_data)
        #instance = serializer.save()
        #write_fuseki = fuseki.FusekiActions()
        #write_fuseki.write(serializer.data)

    #Custom actions when DELETE
    def perform_destroy(self, instance):
        #print("deleted")
        instance.delete()


class Suffix_S5ListAPIView(ListAPIView):
    """
    API view to retrieve list of posts or create new
    """
    permission_classes = [permissions.IsAuthenticatedOrReadOnly]
    serializer_class = Suffix_S5Serializer

    def get_queryset(self):
        bp = self.kwargs['boilerplate']
        sf = self.kwargs['suffixs']
        #print(group_pk)
        queryset_suffix = Suffix_S5.objects.filter(boilerplate_of_s5 = bp, suffix_owner_of_s5 = sf)
        return queryset_suffix

    #Custom actions when POST
    def perform_create(self, serializer):
        serializer.save()
        #print the data of the post
        #write_fuseki = fuseki.FusekiActions()
        #write_fuseki.write(serializer.data)

class Suffix_S5DetailsAPIView(RetrieveUpdateDestroyAPIView):
    """
    API view to retrieve, update or delete post
    """
    permission_classes = [permissions.IsAuthenticatedOrReadOnly]
    serializer_class = Suffix_S5Serializer

    def get_queryset(self):
        bp = self.kwargs['boilerplate']
        sf = self.kwargs['suffixs']
        #print(group_pk)
        queryset_suffix = Suffix_S5.objects.filter(boilerplate_of_s5 = bp, suffix_owner_of_s5 = sf)
        return queryset_suffix

    #Custom actions when PUT
    def perform_update(self, serializer):
        serializer.save()
        #print(self.new.changed_data)
        #instance = serializer.save()
        #write_fuseki = fuseki.FusekiActions()
        #write_fuseki.write(serializer.data)

    #Custom actions when DELETE
    def perform_destroy(self, instance):
        #print("deleted")
        instance.delete()


class Suffix_S6ListAPIView(ListAPIView):
    """
    API view to retrieve list of posts or create new
    """
    permission_classes = [permissions.IsAuthenticatedOrReadOnly]
    serializer_class = Suffix_S6Serializer

    def get_queryset(self):
        bp = self.kwargs['boilerplate']
        sf = self.kwargs['suffixs']
        #print(group_pk)
        queryset_suffix = Suffix_S6.objects.filter(boilerplate_of_s6 = bp, suffix_owner_of_s6 = sf)
        return queryset_suffix

    #Custom actions when POST
    def perform_create(self, serializer):
        serializer.save()
        #print the data of the post
        #write_fuseki = fuseki.FusekiActions()
        #write_fuseki.write(serializer.data)

class Suffix_S6DetailsAPIView(RetrieveUpdateDestroyAPIView):
    """
    API view to retrieve, update or delete post
    """
    permission_classes = [permissions.IsAuthenticatedOrReadOnly]
    serializer_class = Suffix_S6Serializer

    def get_queryset(self):
        bp = self.kwargs['boilerplate']
        sf = self.kwargs['suffixs']
        #print(group_pk)
        queryset_suffix = Suffix_S6.objects.filter(boilerplate_of_s6 = bp, suffix_owner_of_s6 = sf)
        return queryset_suffix

    #Custom actions when PUT
    def perform_update(self, serializer):
        serializer.save()
        #print(self.new.changed_data)
        #instance = serializer.save()
        #write_fuseki = fuseki.FusekiActions()
        #write_fuseki.write(serializer.data)

    #Custom actions when DELETE
    def perform_destroy(self, instance):
        #print("deleted")
        instance.delete()


